@@ -2,16 +2,17 @@ import json
 from pathlib import Path
 from typing import Type
 
+import box
 import jsonschema
 import torch.nn as nn
-import torch_model_loader
 from mobile_block import MobileBlock
 
+from .batch_norm_2d import load_default_batch_norm_2d
 from .configs import MobileNetV2Config
-from .layers import Flatten, GlobalPooling, InplaceReLU6
+from .layers import Classifier, Squeeze, GlobalPooling, InplaceReLU6
 
 
-@torch_model_loader.register
+@box.register(tag='model')
 class MobileNetV2(nn.Module):
     with open(str(Path(__file__).parent / 'schema' / 'mobilenet_v2_config.json')) as f:
         schema = json.load(f)
@@ -21,27 +22,27 @@ class MobileNetV2(nn.Module):
         t = config.expansion_ratio
         input_size = config.input_size
         width_multiple = config.width_multiple
-        batch_norm_2d: Type[nn.BatchNorm2d] = torch_model_loader.load_default_batch_norm_2d()
-
-        block_settings = [
-            # t, c, n, s
-            [1, 16, 1, 1],
-            [t, 24, 2, 2],
-            [t, 32, 3, 2],
-            [t, 64, 4, 2],
-            [t, 96, 3, 1],
-            [t, 160, 3, 2],
-            [t, 320, 1, 1],
-        ]
+        batch_norm_2d: Type[nn.BatchNorm2d] = load_default_batch_norm_2d()
+        stride_list = config.stride_list
 
         # building first layer
         in_channels = int(32 * width_multiple)
         blocks = [
-            self.conv_bn_relu(in_channels=3, out_channels=in_channels, kernel=3, stride=2, batch_norm_2d=batch_norm_2d)
+            self.conv_bn_relu(3, in_channels, kernel=3, stride=stride_list[0], batch_norm_2d=batch_norm_2d)
         ]
         input_size //= 2
 
         # building inverted residual blocks
+        block_settings = [
+            # t, c, n, s
+            [1, 16, 1, 1],
+            [t, 24, 2, stride_list[1]],
+            [t, 32, 3, stride_list[2]],
+            [t, 64, 4, stride_list[3]],
+            [t, 96, 3, 1],
+            [t, 160, 3, stride_list[4]],
+            [t, 320, 1, 1],
+        ]
         for expansion, channels, times, first_stride in block_settings:
             out_channels = int(channels * width_multiple)
 
@@ -66,9 +67,8 @@ class MobileNetV2(nn.Module):
         last_channels = int(1280 * max(width_multiple, 1.0))
         blocks.append(self.conv_bn_relu(in_channels, last_channels, kernel=1, stride=1, batch_norm_2d=batch_norm_2d))
         blocks.append(GlobalPooling())
-        blocks.append(Flatten())
-        blocks.append(nn.Dropout(config.dropout_ratio))
-        blocks.append(nn.Linear(last_channels, config.feature_dim))
+        blocks.append(Squeeze())
+        blocks.append(Classifier(last_channels, config.feature_dim, dropout=config.dropout_ratio))
 
         self.blocks = nn.Sequential(*blocks)
         self._initialize_weights(batch_norm_2d=batch_norm_2d)
